@@ -3,11 +3,14 @@ import abc
 import os
 import time
 import logging
+from typing_extensions import Self
 
 from zipfile import ZipFile, ZipInfo
 from typing import AsyncIterable, cast
 from io import RawIOBase
 
+from sanic import request
+
 from fpx import exception
 from fpx.model import Ticket
 
@@ -46,23 +49,27 @@ class Pipe(abc.ABC):
     _filename = "download"
 
     @classmethod
-    async def choose(cls, ticket: Ticket):
+    def choose(cls, ticket: Ticket, request: request.Request):
         if ticket.type == "zip":
             pipe = ZipPipe(ticket)
         elif ticket.type == "stream":
-            pipe = StreamPipe(ticket)
+            if request.app.config.FPX_PIPE_SILLY_STREAM:
+                pipe = SillyStreamPipe(ticket)
+            else:
+                pipe = StreamPipe(ticket)
         else:
             raise exception.RequestError(
                 {"type": f"Unsupported ticket type: {ticket.type}"}
             )
-
-        await pipe.prepare()
         return pipe
 
     def __init__(self, ticket: Ticket):
         self.ticket = ticket
 
-    async def prepare(self):
+    async def __aenter__(self) -> Self:
+        return self
+
+    async def __aexit__(self, exc_type, exc, tb):
         pass
 
     def content_type(self):
@@ -80,11 +87,13 @@ class ZipPipe(Pipe):
     _content_type = "application/zip"
     _filename = "collection.zip"
 
-    async def prepare(self):
+    async def __aenter__(self):
         name = self.ticket.options.get("filename")
         if name:
             self._filename = name
 
+        return self
+
     async def chunks(self) -> AsyncIterable[bytes]:
         stream = _Stream()
         with ZipFile(
@@ -113,21 +122,52 @@ class ZipPipe(Pipe):
         yield stream.get()
 
 
+class SillyStreamPipe(Pipe):
+    def __init__(self, ticket: Ticket):
+        super().__init__(ticket)
+        self._content_type = None
+        self._filename = None
+
+    async def __aenter__(self):
+        async for _path, name, _content, resp in utils.stream_downloaded_files(
+            self.ticket.items
+        ):
+            self._filename = name
+            self._content_type = resp.content_type
+            break
+        return self
+
+    async def chunks(self) -> AsyncIterable[bytes]:
+        async for path, name, content, resp in utils.stream_downloaded_files(
+            self.ticket.items
+        ):
+
+            async for chunk in content.iter_chunked(CHUNK_SIZE):
+                yield chunk
+
+
 class StreamPipe(Pipe):
     def __init__(self, ticket: Ticket):
         super().__init__(ticket)
         self._content_type = None
         self._filename = None
 
-    async def prepare(self):
-        name, type_, content = await self._crawl_file()
-        self._filename = name
-        self._content_type = type_
-        self._content = content
+    async def __aenter__(self):
+        self._gen = self._crawl_file()
+        async for name, type_, crawler in self._gen:
+            self._filename = name
+            self._content_type = type_
+            self._content = crawler()
+            break
+        return self
+
+    async def __aexit__(self, exc_type, exc, tb):
+        async for _ in self._gen:
+            pass
 
     async def chunks(self) -> AsyncIterable[bytes]:
         assert self._content
-        async for chunk in self._content():
+        async for chunk in self._content:
             yield cast(bytes, chunk)
 
     async def _crawl_file(self):
@@ -139,6 +179,4 @@ class StreamPipe(Pipe):
                 async for chunk in content.iter_chunked(CHUNK_SIZE):
                     yield chunk
 
-            return name, resp.content_type, crawler
-
-        raise exception.RequestError({"items": "Cannot download a file"})
+            yield name, resp.content_type, crawler
